'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (_ref) {
    var t = _ref.types;

    return {
        visitor: {
            Program: {
                enter: function enter() {
                    done__onamespace = false;
                    done__ofilepath = false;
                }
            },
            Identifier: function Identifier(babelPath, state) {
                var _state$opts = state.opts,
                    filename = _state$opts.filename,
                    childFolderName = _state$opts.childFolderName,
                    separator = _state$opts.separator;

                var oFilename = filename || 'omodule.js';
                var oChildFolderName = childFolderName || 'omodules';
                separator = separator || '/';
                var dirname = getFolderName(state.file.opts.filename);

                if (babelPath.node.name === '__omodule_name') {
                    babelPath.replaceWith(t.stringLiteral(getOmoduleName(dirname, state.file.opts.sourceRoot, oFilename)));
                }
                if (babelPath.node.name === '__omodule_namespace') {
                    var _dirname = getFolderName(state.file.opts.filename);
                    babelPath.replaceWith(t.stringLiteral(getOmoduleNameSpace(_dirname, state.file.opts.sourceRoot, oFilename, separator)));
                }
                if (babelPath.node.name === '__omodule_filename') {
                    babelPath.replaceWith(t.stringLiteral(_path2.default.basename(state.file.opts.filename)));
                }
                if (babelPath.node.name === '__omodule_childnames') {
                    var names = getOmoduleChildNames(state.file.opts.filename, state.file.opts.sourceRoot, oFilename, oChildFolderName);

                    babelPath.replaceWith(t.arrayExpression(names.map(function (name) {
                        return t.stringLiteral(name);
                    })));
                }

                var _state$opts2 = state.opts,
                    rootPath = _state$opts2.rootPath,
                    _state$opts2$namespac = _state$opts2.namespacePrefix,
                    namespacePrefix = _state$opts2$namespac === undefined ? '' : _state$opts2$namespac;

                var absoluteRootPath = void 0;
                if (rootPath) {
                    if (typeof rootPath === 'string') {
                        absoluteRootPath = createAbsolutePath(rootPath);
                    } else {
                        throw new Error('rootPath must be a string. Received ' + (typeof rootPath === 'undefined' ? 'undefined' : _typeof(rootPath)));
                    }
                } else {
                    absoluteRootPath = projectRootPath;
                }

                var absoluteFilename = createAbsolutePath(state.file.opts.filename);

                var matchRootPath = _path2.default.normalize(absoluteRootPath).replace(/[\\\/]+$/, '');

                if (absoluteFilename.indexOf(absoluteRootPath) > -1) {
                    if (babelPath.node.name === '__onamespace' && !done__onamespace) {
                        var s = getONamesapce(matchRootPath, absoluteFilename, namespacePrefix);
                        var prog = babelPath.find(function (path) {
                            return path.isProgram();
                        });
                        var ident = t.identifier('__onamespace');
                        var string = t.stringLiteral(s);
                        var declarator = t.variableDeclarator(ident, string);
                        var declaration = t.variableDeclaration('var', [declarator]);
                        prog.scope.block.body.unshift(declaration);
                        done__onamespace = true;
                    }
                    if (babelPath.node.name === '__ofilepath' && !done__ofilepath) {
                        var _s = getOFilePath(matchRootPath, absoluteFilename);
                        var _prog = babelPath.find(function (path) {
                            return path.isProgram();
                        });
                        var _ident = t.identifier('__ofilepath');
                        var _string = t.stringLiteral(_s);
                        var _declarator = t.variableDeclarator(_ident, _string);
                        var _declaration = t.variableDeclaration('var', [_declarator]);
                        _prog.scope.block.body.unshift(_declaration);
                        done__ofilepath = true;
                    }
                }
            }
        }
    };
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _appRootPath = require('app-root-path');

var _appRootPath2 = _interopRequireDefault(_appRootPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

//// support omodule-namespace --- begin
var projectRootPath = _appRootPath2.default.path;
var OMODULE_CHILDREN_FOLDERNAME = 'omodules';
var SEPARATOR = '/';

function isAbsolutePath(p) {
    return _path2.default.isAbsolute(p);
}

function createAbsolutePath(p) {
    if (isAbsolutePath(p)) {
        return _path2.default.normalize(p);
    } else {
        return _path2.default.normalize(_path2.default.join(projectRootPath, p));
    }
}

function getFolderName(p) {
    return _path2.default.join(p, '..');
}

function getONamesapce(matchRootPath, absoluteFilename, namespacePrefix) {
    var onamespace = void 0;
    if (matchRootPath === getFolderName(absoluteFilename)) {
        onamespace = namespacePrefix + '/';
    } else {
        var subPath = absoluteFilename.replace(matchRootPath, '');
        var onamespaceResult = subPath.split('' + _path2.default.sep + OMODULE_CHILDREN_FOLDERNAME + _path2.default.sep).reduce(function (acc, curr) {
            if (acc.done) {
                return _extends({}, acc);
            }

            if (curr) {
                if (curr.indexOf(_path2.default.sep) > -1) {
                    var _onamespace = acc.onamespace + curr.split(_path2.default.sep)[0] + '/';
                    return {
                        onamespace: _onamespace,
                        done: true
                    };
                } else {
                    return _extends({}, acc, { onamespace: acc.onamespace + curr + '/' });
                }
            } else {
                return _extends({}, acc, { onamespace: '/' });
            }
        }, {
            onamespace: '',
            done: false
        });
        onamespace = namespacePrefix + onamespaceResult.onamespace;
    }
    return onamespace.length > 1 ? onamespace.replace(/\/$/, '') : onamespace;
}
function getOFilePath(matchRootPath, absoluteFilename) {
    return absoluteFilename.replace(matchRootPath + _path2.default.sep, '');
}

var done__onamespace = false;
var done__ofilepath = false;

//// support omodule-namespace --- end


var isFile = function isFile(pathString) {
    return _fs2.default.lstatSync(pathString).isFile();
};

var isDirectory = function isDirectory(pathString) {
    return _fs2.default.lstatSync(pathString).isDirectory();
};

var isRoot = function isRoot(dirname, sourceRoot) {
    if (!dirname) {
        return true;
    }
    if (sourceRoot && sourceRoot === dirname) {
        return true;
    }
    var parentDir = getFolderName(dirname);
    if (parentDir === dirname) {
        return true;
    }

    return false;
};

var hasOmoduleFileFun = function hasOmoduleFileFun(oFilename) {
    return function (dirname) {
        var files = _fs2.default.readdirSync(dirname);
        return files.indexOf(oFilename) > -1 && isFile(_path2.default.join(dirname, oFilename));
    };
};

var getOmoduleFolderPath = function getOmoduleFolderPath(dirname, sourceRoot, oFilename) {
    var hasOmoduleFile = hasOmoduleFileFun(oFilename);
    if (hasOmoduleFile(dirname)) {
        return dirname;
    } else {
        var parentDir = getFolderName(dirname);
        if (isRoot(parentDir, sourceRoot)) {
            throw new Error('file: ' + _path2.default.join(dirname, oFilename) + ' not be included in any \'omodule\'.');
        }
        return getOmoduleFolderPath(parentDir, sourceRoot, oFilename);
    }
};

var getOmoduleName = function getOmoduleName(dirname, sourceRoot, oFilename) {
    return _path2.default.basename(getOmoduleFolderPath(dirname, sourceRoot, oFilename));
};

var getParentOmoduleFolderPath = function getParentOmoduleFolderPath(omoduleFolderPath, sourceRoot, oFilename) {
    var hasOmoduleFile = hasOmoduleFileFun(oFilename);
    var omodulesFloderPath = getFolderName(omoduleFolderPath);
    var parentDir = getFolderName(omodulesFloderPath);

    if (isRoot(parentDir, sourceRoot)) {
        return null;
    } else {
        if (hasOmoduleFile(parentDir)) {
            return parentDir;
        } else {
            return null;
        }
    }
};

var getOmoduleNameSpace = function getOmoduleNameSpace(dirname, sourceRoot, oFilename, separator) {
    var subNamespace = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : '';

    var omoduleFolderPath = getOmoduleFolderPath(dirname, sourceRoot, oFilename);
    var omoduleFolderName = _path2.default.basename(omoduleFolderPath);
    var parentOmoduleFolderPath = getParentOmoduleFolderPath(omoduleFolderPath, sourceRoot, oFilename);
    var namespace = subNamespace ? omoduleFolderName + separator + subNamespace : omoduleFolderName;
    if (parentOmoduleFolderPath) {
        return getOmoduleNameSpace(parentOmoduleFolderPath, sourceRoot, oFilename, separator, namespace);
    }

    return namespace;
};

var getOmoduleChildNames = function getOmoduleChildNames(filename, sourceRoot, oFilename, oChildFolderName) {
    var hasOmoduleFile = hasOmoduleFileFun(oFilename);
    var dirname = getFolderName(filename);
    var omoduleFolderPath = getOmoduleFolderPath(dirname, sourceRoot, oFilename);
    var files = _fs2.default.readdirSync(omoduleFolderPath) || [];

    var childOmoduleFolderPath = _path2.default.join(omoduleFolderPath, oChildFolderName);
    if (files.indexOf(oChildFolderName) > -1 && isDirectory(childOmoduleFolderPath)) {
        var childOmoduleFolderFiles = _fs2.default.readdirSync(childOmoduleFolderPath) || [];
        return childOmoduleFolderFiles.reduce(function (names, filename) {
            var filePath = _path2.default.join(childOmoduleFolderPath, filename);
            if (isDirectory(filePath) && hasOmoduleFile(filePath)) {
                return [].concat(_toConsumableArray(names), [filename]);
            } else {
                return names;
            }
        }, []);
    } else {
        return [];
    }
};